<?php

namespace Intuit\Providers\Socialite;

use Illuminate\Support\Arr;
use Illuminate\Support\Facades\Auth;
use Laravel\Socialite\Two\InvalidStateException;
use Laravel\Socialite\Two\ProviderInterface;
use Laravel\Socialite\Two\AbstractProvider;
use Laravel\Socialite\Two\User as SocialUser;

class IntuitProvider extends AbstractProvider implements ProviderInterface
{
	private const hosts = [
		'authorise' => 'https://appcenter.intuit.com/connect/oauth2',
		'tokenendpoint' => 'https://oauth.platform.intuit.com/oauth2/v1/tokens/bearer',
		'local' => 'https://sandbox-quickbooks.api.intuit.com',
		'production' => 'https://quickbooks.api.intuit.com',
	];
	protected $scopes = ['com.intuit.quickbooks.accounting'];

	/* ABSTRACT */

	protected function getAuthUrl($state)
	{
		return $this->buildAuthUrlFromBase(
			Arr::get(self::hosts,'authorise'),
			$state
		);
	}

	protected function getUserByToken($token)
	{
		$response = [];

		$access = $this->getAccessTokenResponse($token);

		if ((! $x=Arr::get($access,'token_type')) || ($x !== 'bearer')) {
			$response['reject'] = TRUE;
			$response['reject_reason'] = 'No bearer token?';

			return $response;
		}

		return $response ?: $access;
	}

	protected function getTokenUrl()
	{
		return Arr::get(self::hosts,'tokenendpoint');
	}

	protected function mapUserToObject(array $user)
	{
		return (new SocialUser)->setRaw(Auth::user()->toArray());
	}

	/* OVERRIDES */

	public function user()
	{
		if ($this->hasInvalidState()) {
			throw new InvalidStateException;
		}

		if (! $this->getCode())
			return NULL;

		$details = $this->getUserByToken($this->getCode());

		if ((! $details) || Arr::get($details,'reject'))
			abort(403, sprintf('Authentication Failed with %s',Arr::get($details,'reject_reason','*no reason given*')));

		$user = $this->mapUserToObject($details);
		$user->refresh_token_expires_in = Arr::get($details,'x_refresh_token_expires_in');
		$user->realmid = $this->request->get('realmId');

		return $user->setToken(Arr::get($details,'access_token'))
			->setRefreshToken(Arr::get($details,'refresh_token'))
			->setExpiresIn(Arr::get($details,'expires_in'));
	}
}